@@ -1,16 +1,16 @@
 ## Copyright (C) 2013,2014 Markus Bergholz
 ## Parts Copyright (C) 2013,2014 Philip Nienhuis
-## 
+##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
 ## the Free Software Foundation; either version 3 of the License, or
 ## (at your option) any later version.
-## 
+##
 ## This program is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
 ## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ## GNU General Public License for more details.
-## 
+##
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
 ## <http://www.gnu.org/licenses/>.
@@ -53,12 +53,13 @@
 ## 2014-06-15 (MB) Fix out-of-bounds error in string cell handling
 ## 2014_08_25 (PRN) Switch to col2num (.oct) for colstr->col conversion to allow
 ##            compilation on 64-bit MingW systems (mex won't compile int64_t)
+## 2014_10_02 (MB) bugfix #43331
 
 function [ raw, xls, rstatus ] = __OCT_xlsx2oct__ (xls, wsh, crange='', spsh_opts)
 
   ## spsh_opts is guaranteed to be filled by caller
 
-  ## If a worksheet if given, check if it's given by a name (string) or a number 
+  ## If a worksheet if given, check if it's given by a name (string) or a number
   if (ischar (wsh))
     # fid = fopen (sprintf ('%s/xl/workbook.xml', xls.workbook));
     # if (fid < 0)
@@ -70,7 +71,9 @@ function [ raw, xls, rstatus ] = __OCT_xlsx2oct__ (xls, wsh, crange='', spsh_opt
       # fclose (fid);
 
       ## Search for requested sheet name
-      id = strmatch (wsh, xls.sheets.sh_names);
+      ## id = strmatch (wsh, xls.sheets.sh_names);
+      ## strmatch is scheduled for deprecation.
+      id = find(strcmp(xls.sheets.sh_names, wsh));
       if (isempty (id))
         error ("xls2oct: cannot find sheet '%s' in file %s", wsh, xls.filename);
       else
@@ -85,7 +88,14 @@ function [ raw, xls, rstatus ] = __OCT_xlsx2oct__ (xls, wsh, crange='', spsh_opt
 
   ## Prepare to open requested worksheet file in subdir xl/ . Note: Win accepts forward slashes
   rawsheet = fopen (sprintf ('%s/xl/worksheets/sheet%d.xml', xls.workbook, wsh));
-  if (rawsheet > 0)
+  if (rawsheet <= 0)
+    # try to open sheet from r:id in worksheets.rels.xml
+    wsh       = xls.sheets.rels( xls.sheets.rels(:,1) == id ,2);
+    rawsheet  = fopen (sprintf ('%s/xl/worksheets/sheet%d.xml', xls.workbook, wsh));
+    if (rawsheet <= 0)
+      error ("Couldn't open worksheet xml file sheet%d.xml\n", wsh);
+    endif
+  else
     ## Get data
     rawdata = fread (rawsheet, "char=>char").';
     fclose (rawsheet);
@@ -98,8 +108,6 @@ function [ raw, xls, rstatus ] = __OCT_xlsx2oct__ (xls, wsh, crange='', spsh_opt
       ## No sharedStrings.xml; implies no "fixed" strings (computed strings can still be there)
       strings = "";
     end_try_catch
-  else
-    error ("Couldn't open worksheet xml file sheet%d.xml\n", wsh);
   endif
   rstatus = 0;
 
@@ -267,7 +275,7 @@ function [ raw, xls, rstatus ] = __OCT_xlsx2oct__ (xls, wsh, crange='', spsh_opt
       return
     endif
 
-    ## 4. Prepare 
+    ## 4. Prepare
     ## Get the row number (currently supported from 1 to 999999)
     vi.row = str2double (cell2mat (regexp (valraw, '(\d+|\d+\d+|\d+\d+\d+|\d+\d+\d+\d+|\d+\d+\d+\d+\+d|\d+\d+\d+\d+\d+\d+)?', "match"))')';
 
@@ -329,7 +337,7 @@ function [ raw, xls, rstatus ] = __OCT_xlsx2oct__ (xls, wsh, crange='', spsh_opt
     cc = idx.mincol - 1;
     raw = raw(xls.limits(2, 1)-rc : xls.limits(2, 2)-rc, xls.limits(1, 1)-cc : xls.limits(1, 2)-cc);
   endif
- 
+
   if (! isempty (val))
     rstatus = 1;
   endif
