@@ -1,4 +1,4 @@
-## Copyright (C) 2009,2010,2011,2012,2013 Philip Nienhuis <prnienhuis at users.sf.net>
+## Copyright (C) 2009,2010,2011,2012,2013,2014 Philip Nienhuis
 ## 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -24,20 +24,21 @@
 ##
 ## Calling xlsopen without specifying a return argument is fairly useless!
 ##
-## To make this function work at all, you need MS-Excel (95 - 2013), and/or
-## the Java package >= 1.2.8 plus Apache POI >= 3.5 and/or JExcelAPI and/or
-## OpenXLS and/or OpenOffice.org (or clones) installed on your computer +
-## proper javaclasspath set. These interfaces are referred to as COM, POI,
+## xlsopen works with interfaces, which are links to external software.
+## For I/O from/to OOXML (Excel 2007 and up), ODS 1.2 and Gnumeric, no
+## additional software is required when the OCT interface is used (see below).
+## For all other spreadsheet formats, you need one or more of MS-Excel
+## (95 - 2013), or a Java JRE plus Apache POI >= 3.5 and/or JExcelAPI
+## and/or OpenXLS and/or OpenOffice.org (or clones) installed on your computer
+## + proper javaclasspath set. These interfaces are referred to as COM, POI,
 ## JXL, OXS, and UNO, resp., and are preferred in that order by default
-## (depending on their presence).
-## For OOXML support, in addition to Apache POI support you also need the
-## following jars in your javaclasspath: poi-ooxml-schemas-3.5.jar,
-## xbean.jar and dom4j-1.6.1.jar (or later versions). Later OpenOffice.org
-## versions (UNO) have support for OOXML as well.
-## Excel'95 spreadsheets can only be read by JExcelAPI and OpenOffice.org.
-## For just reading OOXML (.xlsx or .xlsm), no Java or add-on packages are 
-## required; but currently you loose a bit of the flexibility of the other
-## interfaces.
+## (depending on their presence). Currently the OCT interface has the lowest
+## priority as it is still experimental.
+## For OOXML read/write support in principle no additional SW is needed.
+## However, the COM, POI and UNO interfaces may provide better OOXML write
+## performance and/or more flexibility.
+## Excel'95 spreadsheets (BIFF5) can only be read using the COM (Excel-ActiveX),
+## JXL (JExcelAPI), and UNO (Open-/LibreOffice) interfaces.
 ##
 ## @var{filename} should be a valid .xls or xlsx Excel file name (including
 ## extension). But if you use the COM interface you can specify any extension
@@ -46,15 +47,15 @@
 ## allowed. If @var{filename} does not contain any directory path, the file
 ## is saved in the current directory.
 ##
-## If @var{readwrite} is set to 0 (default value) or omitted, the Excel file
-## is opened for reading. If @var{readwrite} is set to True or 1, an Excel
-## file is opened (or created) for reading & writing.
+## If @var{readwrite} is set to 0 (default value) or omitted, the spreadsheet
+## file is opened for reading. If @var{readwrite} is set to true or 1, a
+## spreadsheet file is opened (or created) for reading & writing.
 ##
 ## Optional input argument @var{reqintf} can be used to override the Excel
 ## interface that otherwise is automatically selected by xlsopen. Currently
 ## implemented interfaces (in order of preference) are 'COM' (Excel/COM),
-## 'POI' (Java/Apache POI), 'JXL' (Java/JExcelAPI), 'OXS' (Java/OpenXLS), or
-## 'UNO' (Java/OpenOffice.org - EXPERIMENTAL!).
+## 'POI' (Java/Apache POI), 'JXL' (Java/JExcelAPI), 'OXS' (Java/OpenXLS),
+## 'UNO' (Java/OpenOffice.org - EXPERIMENTAL!), or 'OCT' (native Octave).
 ## In most situations this parameter is unneeded as xlsopen automatically
 ## selects the most useful interface present.
 ##
@@ -77,7 +78,7 @@
 ##
 ## @end deftypefn
 
-## Author: Philip Nienhuis
+## Author: Philip Nienhuis <prnienhuis at users.sf.net>
 ## Created: 2009-11-29
 ## Updates:
 ## 2010-01-03 Added OOXML support
@@ -128,8 +129,19 @@
 ##     ''     Adapted header to OCT (also Excel 2013 is supported)
 ## 2013-10-01 Some adaptations for gnumeric
 ## 2013-10-20 Overhauled file extension detection logic
-## 2013-11-03 Improved interface selection (fix a.0., fallback to JXL for xlsx)
+## 2013-11-03 Improved interface selection (fix a.o., fallback to JXL for xlsx)
 ## 2013-11-04 Catch attempts to write with only OCT interface
+## 2013-12-01 Add support for ODS (Excel 2007+ and OpenOffice.org/LibreOffice support it)
+## 2013-12-27 Use one variable for processed file type
+##     ''     Style fixes
+## 2013-12-28 Allow OOXML support for OpenXLS
+## 2014-01-01 Add .csv to supported file extensions
+##     ''     Add warning that UNO will write ODS f. unsupported file extensions
+##     ''     Copyright string update
+## 2014-02-02 Allow write support for OCT interface
+## 2014-04-13 Updated texinfo header
+## 2014-04-26 Fix error messages (no more traceback)
+## 2014-04-30 Allow gnumeric write support (remove catch)
 
 function [ xls ] = xlsopen (filename, xwrite=0, reqinterface=[])
 
@@ -150,20 +162,21 @@ function [ xls ] = xlsopen (filename, xwrite=0, reqinterface=[])
       usage ("XLS = xlsopen (Xlfile [, Rw] [, reqintf]). But no return argument specified!"); 
   endif
 
-  if (~(islogical (xwrite) || isnumeric (xwrite)))
+  if (! (islogical (xwrite) || isnumeric (xwrite)))
       usage ("xlsopen.m: numerical or logical value expected for arg ## 2 (readwrite)")
   endif
 
-  if (~isempty (reqinterface))
-    if ~(ischar (reqinterface) || iscell (reqinterface))
+  if (! isempty (reqinterface))
+    if (! (ischar (reqinterface) || iscell (reqinterface)))
       usage ("Arg ## 3 (interface) not recognized - character value required"); 
     endif
     ## Turn arg3 into cell array if needed
-    if (~iscell (reqinterface))
+    if (! iscell (reqinterface))
       reqinterface = {reqinterface}; 
     endif
     ## Check if previously used interface matches a requested interface
-    if (isempty (regexpi (reqinterface, lastintf, 'once'){1}))
+    if (isempty (regexpi (reqinterface, lastintf, 'once'){1}) ||
+        ! xlsinterfaces.(upper (reqinterface{1})))
       ## New interface requested
       xlsinterfaces.COM = 0; xlsinterfaces.POI = 0; xlsinterfaces.JXL = 0;
       xlsinterfaces.OXS = 0; xlsinterfaces.UNO = 0; xlsinterfaces.OCT = 0;
@@ -193,7 +206,7 @@ function [ xls ] = xlsopen (filename, xwrite=0, reqinterface=[])
       ## Well, is/are the requested interface(s) supported on the system?
       xlsintf_cnt = 0;
       for ii=1:numel (reqinterface)
-        if (~xlsinterfaces.(toupper (reqinterface{ii})))
+        if (! xlsinterfaces.(toupper (reqinterface{ii})))
           ## No it aint
           printf ("%s is not supported.\n", upper (reqinterface{ii}));
         else
@@ -201,7 +214,7 @@ function [ xls ] = xlsopen (filename, xwrite=0, reqinterface=[])
         endif
       endfor
       ## Reset interface check indicator if no requested support found
-      if (~xlsintf_cnt)
+      if (! xlsintf_cnt)
         chkintf = [];
         xls = [];
         return
@@ -210,20 +223,31 @@ function [ xls ] = xlsopen (filename, xwrite=0, reqinterface=[])
   endif
 
   ## Check if Excel file exists. First check for (supported) file name suffix:
-  ch1 = chk2 = chk5 = 0;
+  ftype = 0;
   has_suffix = 1;
-  [sfxpos, ~, ~, ext] = regexp (filename, '(\.xlsx?|\.gnumeric)');
+  [sfxpos, ~, ~, ext] = regexpi (filename, '(\.xlsx?|\.gnumeric|\.ods|\.csv)');
   if (! isempty (sfxpos))
-    ext = ext{end};
+    ext = lower (ext{end});
     ## .xls or .xls[x,m,b] or .gnumeric is there, but at the right(most) position?
     if (sfxpos(end) <= length (filename) - length (ext))
       ## Apparently not, or it is an unrecognized extension
       ## If xwrite = 0, check file suffix, else add .xls
       has_suffix = 0;
     else
-      chk1 = strcmpi (ext, ".xls");               ## Regular (binary) BIFF 
-      chk2 = strcmpi (ext(1:end-1), ".xls");      ## Zipped XML / OOXML. Catches xlsx, xlsb, xlsm
-      chk5 = strcmpi (ext, ".gnumeric");          ## Zipped XML / gnumeric
+      switch ext
+        case ".xls"                               ## Regular (binary) BIFF
+          ftype = 1;
+        case {".xlsx", ".xlsm", ".xlsb"}          ## Zipped XML / OOXML. Catches xlsx, xlsb, xlsm
+          ftype = 2;
+        case "ods"                                ## ODS 1.2 (Excel 2007+ & OOo/LO can read ODS)
+          ftype = 3;
+        case ".gnumeric"                          ## Zipped XML / gnumeric
+          ftype = 5;
+        case ".csv"                               ## csv. Detected for xlsread afficionados
+          ftype = 6;
+        otherwise
+          warning ("xlsopen: file type ('%s' extension) not supported", ext);
+      endswitch
     endif
   else
     has_suffix = 0;
@@ -234,14 +258,10 @@ function [ xls ] = xlsopen (filename, xwrite=0, reqinterface=[])
   
   ## Adapt file open mode for readwrite argument
   if (xwrite)
-    ## Catch attempts to write gnumeric
-    if (chk5)
-      error ("There's only read support for gnumeric files");
-    endif
-    ## Catch attempts to write xlsx if only OCT interface is supported
-    if (xlsintf_cnt == 1 && xlsinterfaces.OCT)
-      error ("Only the OCT interface is present or requested, but that has only read support");
-    endif
+%    ## Catch attempts to write gnumeric
+%    if (ftype == 5)
+%      error ("xlsopen: there's only read support for gnumeric files\n");
+%    endif
     fmode = 'r+b';
     if (! has_suffix)
       ## Add .xls suffix to filename (all Excel versions can write this)
@@ -264,7 +284,8 @@ function [ xls ] = xlsopen (filename, xwrite=0, reqinterface=[])
   endif
   fid = fopen (filename, fmode);
   if (fid < 0)                      ## File doesn't exist...
-    if (~xwrite)                    ## ...which obviously is fatal for reading...
+    if (! xwrite)                   ## ...which obviously is fatal for reading...
+      ## FIXME process open apps (Excel, LibreOffice, etc) before hard crash
       error ( sprintf ("xlsopen.m: file %s not found\n", filename));
     else                            ## ...but for writing, we need more info:
       fid = fopen (filename, 'rb'); ## Check if it exists at all...
@@ -297,31 +318,36 @@ function [ xls ] = xlsopen (filename, xwrite=0, reqinterface=[])
   xlssupport = 0;
 
   ## Interface preference order is defined below: currently COM -> POI -> JXL -> OXS -> UNO -> OCT
-  ## chk1, chk2 (xls file type) and chk5 (gnumeric) are conveyed depending on interface capabilities
+  ## ftype (file type) is conveyed depending on interface capabilities
 
-  if ((! xlssupport) && xlsinterfaces.COM && (! chk5))
+  if ((! xlssupport) && xlsinterfaces.COM && (ftype != 5))
     ## Excel functioning has been tested above & file exists, so we just invoke it.
     [ xls, xlssupport, lastintf ] = __COM_spsh_open__ (xls, xwrite, filename, xlssupport);
   endif
 
-  if ((! xlssupport) && xlsinterfaces.POI && (chk1 || chk2) && (! chk5))
-    [ xls, xlssupport, lastintf ] = __POI_spsh_open__ (xls, xwrite, filename, xlssupport, chk1, chk2, xlsinterfaces);
+  if ((! xlssupport) && xlsinterfaces.POI && (ftype <= 2))
+    [ xls, xlssupport, lastintf ] = __POI_spsh_open__ (xls, xwrite, filename, xlssupport, ftype, xlsinterfaces);
   endif
 
-  if ((! xlssupport) && xlsinterfaces.JXL && chk1 && (! chk5))
-    [ xls, xlssupport, lastintf ] = __JXL_spsh_open__ (xls, xwrite, filename, xlssupport, chk1);
+  if ((! xlssupport) && xlsinterfaces.JXL && ftype == 1)
+    [ xls, xlssupport, lastintf ] = __JXL_spsh_open__ (xls, xwrite, filename, xlssupport, ftype);
   endif
 
-  if ((! xlssupport) && xlsinterfaces.OXS && chk1 && (! chk5))
-    [ xls, xlssupport, lastintf ] = __OXS_spsh_open__ (xls, xwrite, filename, xlssupport, chk1);
+  if ((! xlssupport) && xlsinterfaces.OXS && ftype <= 2)
+    [ xls, xlssupport, lastintf ] = __OXS_spsh_open__ (xls, xwrite, filename, xlssupport, ftype);
   endif
 
-  if ((! xlssupport) && xlsinterfaces.UNO && (! chk5))
+  if ((! xlssupport) && xlsinterfaces.UNO && (ftype != 5))
+    ## Warn for LO / OOo stubbornness
+    if (ftype == 0 || ftype == 5 || ftype == 6)
+      warning ("UNO interface will write ODS format for unsupported file extensions\n")
+    endif
     [ xls, xlssupport, lastintf ] = __UNO_spsh_open__ (xls, xwrite, filename, xlssupport);
   endif
 
-  if ((! xlssupport) && xlsinterfaces.OCT && (chk2 || chk5))
-    [ xls, xlssupport, lastintf ] = __OCT_spsh_open__ (xls, xwrite, filename, xlssupport, chk2, 0, chk5);
+  if ((! xlssupport) && xlsinterfaces.OCT && ...
+      (ftype == 2 || ftype == 3 || ftype == 5))
+    [ xls, xlssupport, lastintf ] = __OCT_spsh_open__ (xls, xwrite, filename, xlssupport, ftype);
   endif
 
   ## if 
@@ -329,14 +355,18 @@ function [ xls ] = xlsopen (filename, xwrite=0, reqinterface=[])
   ## endif
 
   ## Rounding up. If none of the xlsinterfaces is supported we're out of luck.
-  if (~xlssupport)
+  if (! xlssupport)
     if (isempty (reqinterface))
-      ## This message is appended after message from getxlsinterfaces()
-      printf ("None.\n");
-      warning ("xlsopen.m: no support for spreadsheet I/O"); 
+      ## If no suitable interface was detected (COM or UNO can read .csv), handle
+      ## .csv in xlsread (as that's where Matlab n00bs would expect .csv support)
+      if (ftype != 6)
+        ## This message is appended after message from getxlsinterfaces()
+        printf ("None.\n");
+        warning ("xlsopen.m: no support for spreadsheet I/O\n");
+      endif
     else
       ## No match between filte type & interface found
-      warning ("xlsopen.m: file type not supported by %s %s %s %s %s %s", reqinterface{:});
+      warning ("xlsopen.m: file type not supported by %s %s %s %s %s %s\n", reqinterface{:});
     endif
     xls = [];
     ## Reset found interfaces for re-testing in the next call. Add interfaces if needed.
@@ -349,7 +379,9 @@ function [ xls ] = xlsopen (filename, xwrite=0, reqinterface=[])
     ## xls.changed = 0 (existing/only read from), 1 (existing/data added), 2 (new,
     ## data added) or 3 (pristine, no data added).
     ## Until something was written to existing files we keep status "unchanged".
-    if (xls.changed == 1), xls.changed = 0; endif
+    if (xls.changed == 1)
+      xls.changed = 0; 
+    endif
   endif
 
 endfunction
